/*
 * SubtreeJumpTable.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#include "SubtreeJumpTable.h"

#include <exception/AlibException.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <tree/ranked/PrefixRankedBarPattern.h>
#include <alphabet/RankedSymbol.h>

#include <map>

namespace arbology {

namespace exact {

std::vector < int > SubtreeJumpTable::compute ( const tree::RankedTreeWrapper & subject ) {
	return getInstance ( ).dispatch ( subject.getData ( ) );
}

std::vector < int > SubtreeJumpTable::compute ( const tree::PrefixRankedBarTree & subject ) {
	std::vector < int > res;

	buildDataPointers ( res, subject, 0 );

	return res;
}

/**
 * used to compute subtree jump table.
 * @param begin - index of a root node of a complete subtree to process
 * @return index, increased by one, of the last node in the subtree starting at index begin
 */
int SubtreeJumpTable::buildDataPointers ( std::vector < int > & res, const tree::PrefixRankedBarTree & subject, int begin ) {
	res.push_back ( 0 );
	int index = begin + 1;

	if ( subject.getContent ( )[begin].getSymbol ( ) != subject.getBarSymbol ( ) )
		for ( unsigned i = 0; i < subject.getContent ( )[begin].getRank ( ).getData ( ); i++ )
			index = buildDataPointers ( res, subject, index );

	index++;
	res[begin] = index;
	res.push_back ( begin - 1 );
	return index;
}

auto SubtreeSizesPrefixRankedBarTree = SubtreeJumpTable::RegistratorWrapper < std::vector < int >, tree::PrefixRankedBarTree > ( SubtreeJumpTable::getInstance ( ), SubtreeJumpTable::compute );

} /* namespace exact */

} /* namespace arbology */
